@@ -461,7 +461,14 @@ def relabel_sequential_lineage(y, lineage):
 
         # Fix daughters
         daughters = lineage[cell_id]['daughters']
-        new_lineage[new_cell_id]['daughters'] = [fw[d] for d in daughters]
+        new_lineage[new_cell_id]['daughters'] = []
+        for d in daughters:
+            new_daughter = fw[d]
+            if not new_daughter:  # missing labels get mapped to 0
+                warnings.warn('Cell {} has daughter {} which is not found '
+                              'in the label image `y`.'.format(cell_id, d))
+            else:
+                new_lineage[new_cell_id]['daughters'].append(new_daughter)
 
         # Fix frames
         y_true = np.sum(y == cell_id, axis=(1, 2))
@@ -471,32 +478,82 @@ def relabel_sequential_lineage(y, lineage):
     return y_relabel, new_lineage
 
 
-def is_valid_lineage(lineage):
+def is_valid_lineage(y, lineage):
     """Check if a cell lineage of a single movie is valid.
 
     Daughter cells must exist in the frame after the parent's final frame.
 
     Args:
+        y (numpy.array): The 3D label mask.
         lineage (dict): The cell lineages for a single movie.
 
     Returns:
         bool: Whether or not the lineage is valid.
     """
+    all_cells = np.unique(y)
+    all_cells = set([c for c in all_cells if c])
+    # every cell in the movie should be in the lineage
+    for cell in all_cells:
+        if cell not in lineage:
+            warnings.warn('Cell {} not found in lineage'.format(cell))
+            return False
+
+    # every lineage should have valid fields
     for cell_label, cell_lineage in lineage.items():
         # Get last frame of parent
+        if cell_label not in all_cells:
+            warnings.warn('Cell {} not found in the label image.'.format(
+                cell_label))
+            return False
+
+        # validate `frames`
+        y_true = np.sum(y == cell_label, axis=(1, 2))
+        y_index = np.where(y_true > 0)[0]
+        frames = list(y_index)
+        if frames != cell_lineage['frames']:
+            warnings.warn('Cell {} has invalid frames'.format(cell_label))
+            return False
+
         last_parent_frame = cell_lineage['frames'][-1]
 
         for daughter in cell_lineage['daughters']:
-            try:
-                # get first frame of daughter
-                first_daughter_frame = lineage[daughter]['frames'][0]
-            except KeyError:
+            if daughter not in all_cells or daughter not in lineage:
                 warnings.warn('lineage {} has invalid daughters: {}'.format(
                     cell_label, cell_lineage['daughters']))
                 return False
 
+            # get first frame of daughter
+            try:
+                first_daughter_frame = lineage[daughter]['frames'][0]
+            except IndexError:  # frames is empty?
+                warnings.warn('Daughter {} has no frames'.format(daughter))
+                return False
+
             # Check that daughter's start frame is one larger than parent end frame
             if first_daughter_frame - last_parent_frame != 1:
+                warnings.warn('lineage {} has daughter {} before parent.'.format(
+                    cell_label, daughter))
+                return False
+
+        # TODO: test parent in lineage
+        parent = cell_lineage.get('parent')
+        if parent:
+            try:
+                parent_lineage = lineage[parent]
+            except KeyError:
+                warnings.warn('Parent {} is not present in the lineage'.format(
+                    cell_lineage['parent']))
+                return False
+            try:
+                last_parent_frame = parent_lineage['frames'][-1]
+                first_daughter_frame = cell_lineage['frames'][0]
+            except IndexError:  # frames is empty?
+                warnings.warn('Cell {} has no frames'.format(parent))
+                return False
+            # Check that daughter's start frame is one larger than parent end frame
+            if first_daughter_frame - last_parent_frame != 1:
+                warnings.warn('lineage {} has daughter {} before parent.'.format(
+                    parent, cell_label))
                 return False
 
     return True  # all cell lineages are valid!
@@ -561,244 +618,3 @@ def get_image_features(X, y, appearance_dim=32):
         'morphologies': morphologies,
         # 'adj_matrix': adj_matrix,
     }
-
-
-def concat_tracks(tracks):
-    """Join an iterable of Track objects into a single dictionary of features.
-
-    Args:
-        tracks (iterable): Iterable of tracks.
-
-    Returns:
-        dict: A dictionary of tracked features.
-
-    Raises:
-        TypeError: ``tracks`` is not iterable.
-    """
-    try:
-        list(tracks)  # check if iterable
-    except TypeError:
-        raise TypeError('concatenate_tracks requires an iterable input.')
-
-    def get_array_of_max_shape(lst):
-        # find max dimensions of all arrs in lst.
-        shape = None
-        size = 0
-        for arr in lst:
-            if shape is None:
-                shape = [0] * len(arr.shape[1:])
-            for i, dim in enumerate(arr.shape[1:]):
-                if dim > shape[i]:
-                    shape[i] = dim
-            size += arr.shape[0]
-        # add batch dimension
-        shape = [size] + shape
-        return np.zeros(shape, dtype='float32')
-
-    # insert small array into larger array
-    # https://stackoverflow.com/a/50692782
-    def paste_slices(tup):
-        pos, w, max_w = tup
-        wall_min = max(pos, 0)
-        wall_max = min(pos + w, max_w)
-        block_min = -min(pos, 0)
-        block_max = max_w - max(pos + w, max_w)
-        block_max = block_max if block_max != 0 else None
-        return slice(wall_min, wall_max), slice(block_min, block_max)
-
-    def paste(wall, block, loc):
-        loc_zip = zip(loc, block.shape, wall.shape)
-        wall_slices, block_slices = zip(*map(paste_slices, loc_zip))
-        wall[wall_slices] = block[block_slices]
-
-    # TODO: these keys must match the Track attributes.
-    track_info = {
-        'appearances': get_array_of_max_shape((t.appearances for t in tracks)),
-        'centroids': get_array_of_max_shape((t.centroids for t in tracks)),
-        'morphologies': get_array_of_max_shape((t.morphologies for t in tracks)),
-        'adj_matrices': get_array_of_max_shape((t.adj_matrices for t in tracks)),
-        'norm_adj_matrices': get_array_of_max_shape(
-            (t.norm_adj_matrices for t in tracks)),
-        'temporal_adj_matrices': get_array_of_max_shape(
-            (t.temporal_adj_matrices for t in tracks))
-    }
-
-    for track in tracks:
-        for k in track_info:
-            feature = getattr(track, k)
-            paste(track_info[k], feature, (0,) * len(feature.shape))
-
-    return track_info
-
-
-class Track(object):  # pylint: disable=useless-object-inheritance
-
-    def __init__(self, path=None, tracked_data=None,
-                 appearance_dim=32, distance_threshold=64):
-        if tracked_data:
-            training_data = tracked_data
-        elif path:
-            training_data = load_trks(path)
-        else:
-            raise ValueError('One of `tracked_data` or `path` is required')
-        self.X = training_data['X'].astype('float32')
-        self.y = training_data['y'].astype('int32')
-        self.lineages = training_data['lineages']
-        self.appearance_dim = appearance_dim
-        self.distance_threshold = distance_threshold
-
-        # Correct lineages
-        self._correct_lineages()
-
-        # Remove bad batches
-        self._remove_invalid_batches()
-
-        # Create feature dictionaries
-        features_dict = self._get_features()
-        self.appearances = features_dict['appearances']
-        self.morphologies = features_dict['morphologies']
-        self.centroids = features_dict['centroids']
-        self.adj_matrices = features_dict['adj_matrix']
-        self.norm_adj_matrices = normalize_adj_matrix(self.adj_matrices)
-        self.temporal_adj_matrices = features_dict['temporal_adj_matrix']
-        self.mask = features_dict['mask']
-        self.track_length = features_dict['track_length']
-
-    def _correct_lineages(self):
-        """Ensure sequential labels for all batches"""
-        new_lineages = []
-        for batch in range(self.y.shape[0]):
-
-            y_relabel, new_lineage = relabel_sequential_lineage(
-                self.y[batch], self.lineages[batch])
-
-            new_lineages.append(new_lineage)
-            self.y[batch] = y_relabel
-
-        self.lineages = new_lineages
-
-    def _remove_invalid_batches(self):
-        """Remove all movies and lineages that are invalid.
-
-        All batches with a daughter cell starting in a frame
-        other than the parent's final frame will be dropped.
-        """
-        bad_batches = set()
-
-        for batch in range(self.y.shape[0]):
-            if not is_valid_lineage(self.lineages[batch]):
-                bad_batches.add(batch)
-
-        new_X = []
-        new_y = []
-        new_lineages = []
-        for batch in range(self.X.shape[0]):
-            if batch not in bad_batches:
-                new_X.append(self.X[batch])
-                new_y.append(self.y[batch])
-                new_lineages.append(self.lineages[batch])
-
-        self.X = np.stack(new_X, axis=0)
-        self.y = np.stack(new_y, axis=0)
-        self.lineages = new_lineages
-
-    def _get_features(self):
-        """
-        Extract the relevant features from the label movie
-        Appearance, morphologies, centroids, and adjacency matrices
-        """
-        max_tracks = get_max_cells(self.y)
-        n_batches = self.X.shape[0]
-        n_frames = self.X.shape[1]
-        n_channels = self.X.shape[-1]
-
-        batch_shape = (n_batches, n_frames, max_tracks)
-
-        appearance_shape = (self.appearance_dim, self.appearance_dim, n_channels)
-
-        appearances = np.zeros(batch_shape + appearance_shape, dtype='float32')
-
-        morphologies = np.zeros(batch_shape + (3,), dtype='float32')
-
-        centroids = np.zeros(batch_shape + (2,), dtype='float32')
-
-        adj_matrix = np.zeros(batch_shape + (max_tracks,), dtype='float32')
-
-        temporal_adj_matrix = np.zeros((n_batches,
-                                        n_frames - 1,
-                                        max_tracks,
-                                        max_tracks,
-                                        3), dtype='float32')
-
-        mask = np.zeros(batch_shape, dtype='float32')
-
-        track_length = np.zeros((n_batches, max_tracks, 2), dtype='int32')
-
-        for batch in range(n_batches):
-            for frame in range(n_frames):
-
-                frame_features = get_image_features(
-                    self.X[batch, frame], self.y[batch, frame],
-                    appearance_dim=self.appearance_dim)
-
-                track_ids = frame_features['labels'] - 1
-                centroids[batch, frame, track_ids] = frame_features['centroids']
-                morphologies[batch, frame, track_ids] = frame_features['morphologies']
-                appearances[batch, frame, track_ids] = frame_features['appearances']
-                mask[batch, frame, track_ids] = 1
-
-                # Get adjacency matrix, cannot filter on track ids.
-                cent = centroids[batch, frame]
-                distance = cdist(cent, cent, metric='euclidean')
-                distance = distance < self.distance_threshold
-                adj_matrix[batch, frame] = distance.astype(np.float32)
-
-            # Get track length and temporal adjacency matrix
-            for label in self.lineages[batch]:
-                # Get track length
-                start_frame = self.lineages[batch][label]['frames'][0]
-                end_frame = self.lineages[batch][label]['frames'][-1]
-
-                track_id = label - 1
-                track_length[batch, track_id, 0] = start_frame
-                track_length[batch, track_id, 1] = end_frame
-
-                # Get temporal adjacency matrix
-                frames = self.lineages[batch][label]['frames']
-
-                # Assign same
-                for f0, f1 in zip(frames[0:-1], frames[1:]):
-                    if f1 - f0 == 1:
-                        temporal_adj_matrix[batch, f0, track_id, track_id, 0] = 1
-
-                # Assign daughter
-                # WARNING: This wont work if there's a time gap between mother
-                # cell disappearing and daughter cells appearing
-                last_frame = frames[-1]
-                daughters = self.lineages[batch][label]['daughters']
-                for daughter in daughters:
-                    daughter_id = daughter - 1
-                    temporal_adj_matrix[batch, last_frame, track_id, daughter_id, 2] = 1
-
-            # Assign different
-            same_prob = temporal_adj_matrix[batch, ..., 0]
-            daughter_prob = temporal_adj_matrix[batch, ..., 2]
-            temporal_adj_matrix[batch, ..., 1] = 1 - same_prob - daughter_prob
-
-            # Identify padding
-            for i in range(temporal_adj_matrix.shape[2]):
-                # index + 1 is the cell label
-                if i + 1 not in self.lineages[batch]:
-                    temporal_adj_matrix[batch, :, i] = -1
-                    temporal_adj_matrix[batch, :, :, i] = -1
-
-        feature_dict = {}
-        feature_dict['adj_matrix'] = adj_matrix
-        feature_dict['appearances'] = appearances
-        feature_dict['morphologies'] = morphologies
-        feature_dict['centroids'] = centroids
-        feature_dict['temporal_adj_matrix'] = temporal_adj_matrix
-        feature_dict['mask'] = mask
-        feature_dict['track_length'] = track_length
-
-        return feature_dict
